/*
*         Portable Batch System (PBS) Software License
* 
* Copyright (c) 1999, MRJ Technology Solutions.
* All rights reserved.
* 
* Acknowledgment: The Portable Batch System Software was originally developed
* as a joint project between the Numerical Aerospace Simulation (NAS) Systems
* Division of NASA Ames Research Center and the National Energy Research
* Supercomputer Center (NERSC) of Lawrence Livermore National Laboratory.
* 
* Redistribution of the Portable Batch System Software and use in source
* and binary forms, with or without modification, are permitted provided
* that the following conditions are met:
* 
* - Redistributions of source code must retain the above copyright and
*   acknowledgment notices, this list of conditions and the following
*   disclaimer.
* 
* - Redistributions in binary form must reproduce the above copyright and 
*   acknowledgment notices, this list of conditions and the following
*   disclaimer in the documentation and/or other materials provided with the
*   distribution.
* 
* - All advertising materials mentioning features or use of this software must
*   display the following acknowledgment:
* 
*   This product includes software developed by NASA Ames Research Center,
*   Lawrence Livermore National Laboratory, and MRJ Technology Solutions.
* 
*         DISCLAIMER OF WARRANTY
* 
* THIS SOFTWARE IS PROVIDED BY MRJ TECHNOLOGY SOLUTIONS ("MRJ") "AS IS" WITHOUT 
* WARRANTY OF ANY KIND, AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
* BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
* FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT ARE EXPRESSLY DISCLAIMED.
* 
* IN NO EVENT, UNLESS REQUIRED BY APPLICABLE LAW, SHALL MRJ, NASA, NOR
* THE U.S. GOVERNMENT BE LIABLE FOR ANY DIRECT DAMAGES WHATSOEVER,
* NOR ANY INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
* OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* 
* This license will be governed by the laws of the Commonwealth of Virginia,
* without reference to its choice of law rules.
*/

#include <pbs_config.h>   /* the master config generated by configure */

#include <sys/types.h>
#include <stdlib.h>
#include "libpbs.h"
#include "list_link.h"
#include "attribute.h"
#include "credential.h"
#include "batch_request.h"
#include "pbs_error.h"
#include "dis.h"

static char ident[] = "@(#) $RCSfile: dec_Resc.c,v $ $Revision: 2.1 $";

/*
 * decode_DIS_rescl() - decode a resource request
 *
 *	Used for resource query, resource reserver, resource free.
 *
 *	The batch_request structure must already exist (be allocated by the
 *	caller.   It is assumed that the header fields (protocol type, 
 *	protocol version, request type, and user name) have been decoded.
 *
 *	Data items are:	signed int	resource handle
 *			unsigned int	count of resource queries
 *		followed by that number of:
 *			string		resource list
 */

int decode_DIS_Rescl(sock, preq)
	int	 sock;
	struct batch_request *preq;
{
	int    ct;
	int    i;
	char **ppc;
	int    rc;

	/* first, the resource handle (even if not used in request) */

	preq->rq_ind.rq_rescq.rq_rhandle = disrsi(sock, &rc);
	if (rc) return rc;

	/* next need to know how many query strings */

	ct = disrui(sock, &rc);
	if (rc) return rc;
	preq->rq_ind.rq_rescq.rq_num = ct;
	if (ct) {
		if ((ppc = (char **)malloc(ct * sizeof (char *))) == 0)
			return PBSE_RMSYSTEM;

		for (i=0; i<ct; i++)
			*(ppc + i) = (char *)0;

        	preq->rq_ind.rq_rescq.rq_list = ppc;
		for (i=0; i<ct; i++) {
			*(ppc+i) = disrst(sock, &rc);
			if (rc)
				break;
		}
	}
		
	return rc;
}
