/*
*         Portable Batch System (PBS) Software License
* 
* Copyright (c) 1999, MRJ Technology Solutions.
* All rights reserved.
* 
* Acknowledgment: The Portable Batch System Software was originally developed
* as a joint project between the Numerical Aerospace Simulation (NAS) Systems
* Division of NASA Ames Research Center and the National Energy Research
* Supercomputer Center (NERSC) of Lawrence Livermore National Laboratory.
* 
* Redistribution of the Portable Batch System Software and use in source
* and binary forms, with or without modification, are permitted provided
* that the following conditions are met:
* 
* - Redistributions of source code must retain the above copyright and
*   acknowledgment notices, this list of conditions and the following
*   disclaimer.
* 
* - Redistributions in binary form must reproduce the above copyright and 
*   acknowledgment notices, this list of conditions and the following
*   disclaimer in the documentation and/or other materials provided with the
*   distribution.
* 
* - All advertising materials mentioning features or use of this software must
*   display the following acknowledgment:
* 
*   This product includes software developed by NASA Ames Research Center,
*   Lawrence Livermore National Laboratory, and MRJ Technology Solutions.
* 
*         DISCLAIMER OF WARRANTY
* 
* THIS SOFTWARE IS PROVIDED BY MRJ TECHNOLOGY SOLUTIONS ("MRJ") "AS IS" WITHOUT 
* WARRANTY OF ANY KIND, AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
* BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
* FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT ARE EXPRESSLY DISCLAIMED.
* 
* IN NO EVENT, UNLESS REQUIRED BY APPLICABLE LAW, SHALL MRJ, NASA, NOR
* THE U.S. GOVERNMENT BE LIABLE FOR ANY DIRECT DAMAGES WHATSOEVER,
* NOR ANY INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
* OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* 
* This license will be governed by the laws of the Commonwealth of Virginia,
* without reference to its choice of law rules.
*/
#include <pbs_config.h>   /* the master config generated by configure */

#include <sys/types.h>
#include <netdb.h>
#include <netinet/in.h>
#include <memory.h>
#include "portability.h"
#include "server_limits.h"
#include "net_connect.h"
#include "pbs_error.h"

#if !defined(H_ERRNO_DECLARED)
extern int h_errno;
#endif

static char ident[] = "@(#) $RCSfile: get_hostaddr.c,v $ $Revision: 2.3 $";

/*
 * get_hostaddr.c - contains functions to provide the internal
 *	internet address for a host and to provide the port
 *	number for a service.
 *
 * get_hostaddr - get internal internet address of a host
 *
 *	Returns a pbs_net_t (unsigned long) containing the network
 *	address in host byte order.  A Null value is returned on
 *	an error.
 */

pbs_net_t get_hostaddr(hostname)
	char *hostname;
{
	static struct in_addr  hostaddr;
	struct hostent *hp;
	extern int pbs_errno;

	hp = gethostbyname(hostname);
	if (hp == (struct hostent *)0) {
		if (h_errno == TRY_AGAIN)
			pbs_errno = PBS_NET_RC_RETRY;
		else
			pbs_errno = PBS_NET_RC_FATAL;
		return ((pbs_net_t)0);
	}
	
	memcpy((void *)&hostaddr, (void *)hp->h_addr_list[0], hp->h_length);
	return ((pbs_net_t)ntohl(hostaddr.s_addr));
}
